/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    mdm_test.c

    An example of MDM main program.

    2009-01-15 CF Lin	new release
	2009-11-01 CF Lin	with MDM dll
	2010-06-17 Peter	add more test cases
*/
#ifdef WIN32
#include <windows.h>
#define mysleep(s) Sleep(s*1000)
#else
#include <unistd.h>
#define mysleep(s) sleep(s)
#endif
#include <string.h>
#ifdef WINCE
#include <tchar.h>
#endif
#include "mdm_api.h"	/* $(Root)/inc */
#include "mdm_message.h"	/* $(Root)/inc */

static char gw_ip[16] = {0};
static char agent_ip[16] = {0};

#if defined(WIN32) || defined(_WIN32_WCE)
#define getopt	_getopt
extern int _getopt (int ac, char **av, const char *opts);
extern char *optarg;
#endif


#if defined(WIN32) && !defined(WINCE)
BOOL WINAPI
ConsoleEventHandler(DWORD dwCtrlType)
{
	BOOL ret;

	switch (dwCtrlType)
	{
	case CTRL_C_EVENT:
		printf("CTRL_C not allowed!\n");
		return TRUE;
	case CTRL_CLOSE_EVENT:
		printf("CLOSE_WINDOW not allowed!\n");
		return TRUE;
	case CTRL_BREAK_EVENT:
		printf("CTRL_BREAK got\n");
		ret = TRUE;
		break;
	case CTRL_LOGOFF_EVENT:
		printf("LOGOFF got\n");
		ret = FALSE;
		break;
	case CTRL_SHUTDOWN_EVENT:
		printf("SHUTDOWN got\n");
		ret = FALSE;
		break;
	}
	mdm_stop();

	return ret;
}
#endif

static void
send_req(MDMCONN *con, char *nick)
{
	FILE *fp;
	char buf[1024];
	char *p;
	int len, tag;

	fp = fopen("cmd.txt", "r");
	if (fp == NULL) {
		printf("no command file\n");
		return;
	}
	while (fgets(buf, sizeof(buf), fp)) {
		if (buf[0] == ';' || buf[0] == '#') continue;
		len = strlen(buf);
		if (buf[len-1] != '\n') continue;
		buf[len-1] = 0;
		/* get message ID */
		p = strchr(buf, ' ');
		if (p == NULL) continue; /* empty line or wrong format */
		*p = 0;
		tag = atoi(buf); p++;
		switch (tag) {
		case MSG_GATEWAY_DLL_NAME_GET:
		case MSG_MXFTP_3TIER_CLI2GWY: /* 3-tier <gwyDir> <fName> <cliDir> */
		case MSG_MXFTP_3TIER_GWY2CLI: /* 3-tier <gwyDir> <fName> <cliDir> */
		case MSG_FMNGR_GATEWAY_BROWSE:
		case MSG_FMNGR_GATEWAY_FILE_EXIST:
		case MSG_FMNGR_GATEWAY_RENAME:
		case MSG_FMNGR_GATEWAY_MKDIR:
		case MSG_FMNGR_GATEWAY_RMDIR:
		case MSG_FMNGR_GATEWAY_DELETE:
		case MSG_FMNGR_GATEWAY_DELETE_ALL:
		case MSG_FMNGR_GATEWAY_CHMOD:
			if (p == NULL) /* no argument */
				send_cmd(con, tag, "");
			else
				send_cmd(con, tag, "%s", p);
			continue;
		case MSG_MXFTP_3TIER_DEV2CLI: /* 3-tier <devNick> <devDir> <fName> <cliDir> */
		case MSG_MXFTP_3TIER_CLI2DEV: /* 3-tier <devNick> <devDir> <fName> <cliDir> */
		case MSG_MXFTP_2TIER_DEV2CGW: /* 2-tier <devNick> <devDir> <fName> <cliDir> */
		case MSG_MXFTP_2TIER_CGW2DEV: /* 2-tier <devNick> <devDir> <fName> <cliDir> */
			send_cmd(con, tag, "%s %s", nick, p);
			continue;
		default:
			break;
		}
		if (p == NULL) /* no argument */
			send_peer(con, nick, tag, "");
		else
			send_peer(con, nick, tag, "%s", p);
	}
	fclose(fp);
}

/*	This function is provided by the GUI and is called when there is
	message coming and is not intercepted by the client.
*/
static void 
mngr_message (MDMCONN *con, int tag, int argc, char **argv)
{
	char *nick;
	int i;
	char ip[32];
	char *p;
#if 1
	printf("******************* mngr_recv [%d]", tag);
	for (i=0; i < argc; i++)
		printf(" %s", argv[i]);
	printf("\n");
#endif
	if (argc && argv) 
		nick = argv[0]; /* it works for most of messages */
	else
		nick = NULL;

	switch(tag)
	{
	/* periodically gets the information of a device: */ 
	case MSG_DEVICE_MEMORY_INFO: /* <nick> <freeRAM> <totalRAM> */
		break;
	/* the status of a upload */ 
	case MSG_MXFTP_UPLOAD_STATUS: /*  <nick> <key> <state> <foffs> <fsize> <path> */
		break;
	/* the status of a download */ 
	case MSG_MXFTP_DNLOAD_STATUS: /* <nick> <key> <state> <foffs> <fsize> <path> */
		break;
	/* a device file is completely uploaded */ 
	case MSG_MXFTP_UPLOAD_COMPLETE: /* <nick> <key> <path> */
		break;
	/* a device file is completely downloaded */ 
	case MSG_MXFTP_DNLOAD_COMPLETE: /* <nick> <path> */
		break;
	/* error in upload/downloaded */ 
	case MSG_MXFTP_ERROR_MESSAGE: /* <nick> <key> <msg> */
		break;
	/* a device logs on to the system */ 
	case MSG_DEVICE_NICK: /* <nick> */ 
		/* check if incoming gateway IP match specified gateway IP */
		if (gw_ip[0] != 0) {
			printf("gw ip=%s, gw nick=%s\n", gw_ip, con->user->nick);
			strcpy(ip, con->user->nick);
			p = strchr(ip, ':');
			if (p != NULL) {
				*p = 0;
				if (strcmp(ip, gw_ip))
					break;
			}
		}
		/* check if incoming agent IP match specified agent IP */
		if (agent_ip[0] != 0) {
			printf("agent ip=%s, agent nick=%s\n", agent_ip, nick);
			strcpy(ip, nick);
			p = strchr(ip, ':');
			if (p != NULL) {
				*p = 0;
				if (strcmp(ip, agent_ip))
					break;
			}
		}
		send_req(con, nick);
		break;
	/* one of the device leaves the system */
	case MSG_DEVICE_LEAVE:  /* <nick> */
		break;
	/* a gateway accepts my connection to it */
	case MSG_GATEWAY_ACCEPT_LOGIN:  /* <gateway> <myNick> */
		printf("gw login nick=%s\n", con->user->nick);
		break;
	/* a gateway disconnected */
	case MSG_GATEWAY_LEAVE:
		printf("gw leave nick=%s\n", con->user->nick);
		break;

    default:
        break;
	}
}

static void
usage(void)
{
	printf("Usage: program [type]\n \
		\t[type] (default agent) : [-g] (gateway) or [-c] (client) or [-a] (agent) or [-c -g] (client-gateway)\n \
		\tor [-h] help\n \
		\t-w gateway_IP -n agent_IP\n\n");
}

int
#ifdef WINCE
_tmain(int argc, _TCHAR **targv)
#else
main (int argc, char **argv)
#endif
{
#ifdef WINCE
	char cmdline[256], *argv[32];
	TCHAR command[MAX_PATH];
	int len;
#endif
	int i;
	unsigned int p_type=0, thread=0;

#ifdef WINCE
	_tcscpy(command, targv[0]);
	for (i = 1; i < argc && i < 32; i++)
	{
		len = _tcslen(command);
		_sntprintf(&command[len], MAX_PATH-len,_T(" %s"), targv[i]);
	}
#ifdef UNICODE
	WideCharToMultiByte(CP_ACP, 0, command, -1, cmdline, 256, NULL, NULL);
#else
    strcpy(cmdline, command);
#endif
	argc = split_line(argv, 32, cmdline);
#endif

	/* parse the arguments */
	while ((i=getopt(argc, argv, "hgcatw:n:s:")) != -1)
	{
		switch(i)
		{
		case 'g':
			p_type |= PROGRAM_TYPE_GATEWAY;
			break;
		case 'a':
			p_type |= PROGRAM_TYPE_DEVICE;
			break;
#if !defined(_WIN32_WCE) && !defined(WIN32)
		case 't':
			p_type |= PROGRAM_TYPE_DLL_TEST;
			break;
#endif
		case 'c':
			p_type |= PROGRAM_TYPE_CLIENT;
			thread |= 1;
			break;
		case 'w':
			if (strlen(optarg) > 15) {
				strncpy(gw_ip, optarg, 15);
				gw_ip[15] = 0;
			} else {
				strcpy(gw_ip, optarg);
			}
			break;
		case 'n':
			if (strlen(optarg) > 15) {
				strncpy(agent_ip, optarg, 15);
				agent_ip[15] = 0;
			} else {
				strcpy(agent_ip, optarg);
			}
			break;
		case 'h':
		default:
			usage();
			return 1;
		}
	}
	if ( !(p_type == (PROGRAM_TYPE_CLIENT|PROGRAM_TYPE_GATEWAY) || 
        p_type == (PROGRAM_TYPE_DEVICE|PROGRAM_TYPE_GATEWAY) ||
		p_type == PROGRAM_TYPE_GATEWAY || p_type == PROGRAM_TYPE_CLIENT ||
		p_type == PROGRAM_TYPE_DEVICE
#if !defined(_WIN32_WCE) && !defined(WIN32)
		|| p_type == (PROGRAM_TYPE_DEVICE | PROGRAM_TYPE_DLL_TEST)
#endif
		|| p_type == 0) )
	{
		usage();
		return 1;
	}

#if defined(WIN32) && !defined(WINCE)
	/* set console control handler */
	if (! SetConsoleCtrlHandler(ConsoleEventHandler, TRUE))
	{
		printf("SetConsoleCtrlHandler() fail\n");
		return -1;
	}
#endif

	/* handle those messages that are not handled by the MDM core */
	if (p_type & PROGRAM_TYPE_CLIENT)
	{
		mdm_set_mngr_function(mngr_message);
	}
	/* start MDM */
	if (mdm_start(p_type, thread) != 0)
	{
		printf("Failed to start MDM\n");
		return -2;
	}
	if (p_type & PROGRAM_TYPE_CLIENT)
	{
		do
		{
			mysleep(2);
		} while (mdm_alive());
	}
	return 0;
}
